****************************************************
* Trapped Factors and China's Impact on Global Growth
* Bloom, Romer, Terry, Van Reenen
* Code to replicate Table 1
* Questions to stephenjamesterry@gmail.com
****************************************************

clear
set matsize 10000

****************************************************
*****PROCESS US TRADE DATA FOR USE WITH TITC DATA
****************************************************

*convert SIC codes to integers in COMTRADE data 
use "COMTRADE_NBER.dta", clear
replace ult_sic = "3140" if ult_sic == "314X"
destring ult_sic, replace
save "COMTRADE_NBER_INT.dta", replace

*load in TITC microdata
use "titc_bdvr_small.dta", clear

save "titc.dta", replace

*merge to US comtrade data
merge m:1 year ult_sic using "COMTRADE_NBER_INT.dta", keep(match)
drop _merge

****************************************************
*****CONSTRUCT TRADE RATIOS & CHGS FROM TITC & US DATA
****************************************************

*now, construct a version of TITC which collapses the trade data to cty x ult_sic x year cells
use "titc.dta", clear

*extract production and chinese imports from TITC data
gen productioncty = newprod
gen chinatocty = value_china

collapse (mean) productioncty chinatocty, by(year ult_sic cty)
save "TITC_aggs.dta", replace

keep if year==1996
gen productionctybase = productioncty
gen chinatoctybase = chinatocty
drop chinatocty year chinatocty

save "TITC_aggs_baseyear.dta", replace
use "TITC_aggs.dta", clear
merge m:1 cty ult_sic using "TITC_aggs_baseyear.dta", keep(match)
drop _merge
save "TITC_aggs.dta", replace

*now, compute US production relative to a base year
use "COMTRADE_NBER_INT.dta", clear
gen productionus = vship*1000000 
save "COMTRADE_NBER_INT.dta", replace
keep if year==1996
gen productionusbase = productionus
gen chinatousbase = chinatous
keep ult_sic chinatousbase productionusbase
save "COMTRADE_NBER_INT_baseyear.dta", replace
use "COMTRADE_NBER_INT.dta", clear
merge m:1 ult_sic using "COMTRADE_NBER_INT_baseyear.dta", keep(match)
drop _merge
merge m:1 year ult_sic using "COMTRADE_NBER_INT.dta", force keep(match)
drop _merge
save "COMTRADE_NBER_INT.dta", replace

*compute import ratios
use "TITC_aggs.dta", clear
merge m:1 ult_sic year using "COMTRADE_NBER_INT.dta", keep(match)
drop _merge

gen chinatoctyfrac = chinatocty/productionctybase
gen chinatousfrac = chinatous/productionusbase

*compute import ratio changes
scalar nlags = 5

sort cty ult_sic year
by cty ult_sic: gen chinatoctychg = chinatoctyfrac - chinatoctyfrac[_n-nlags]
by cty ult_sic: gen chinatouschg = chinatousfrac - chinatousfrac[_n-nlags]

_pctile chinatouschg, p(1 99)
gen lbus = r(r1)
gen ubus = r(r2)

_pctile chinatoctychg, p(1 99)
gen lbcty = r(r1)
gen ubcty = r(r2)

drop if chinatouschg<lbus|chinatouschg>ubus|chinatoctychg<lbcty|chinatoctychg>ubcty

drop lbus lbcty ubus ubcty
save "TITC_IMPORT_RATIOS.dta", replace


****************************************************
*****MERGE TRADE RATIO CHGS + INSTRUMENTS INTO TITC 
****************************************************

*now, merge back in to TITC microdata
use "titc.dta", clear

scalar nlags = 5

sort bvd year
by bvd: gen d5sale =  2*(opre - opre[_n-nlags])/(opre + opre[_n-nlags])
label var d5sale "5-year sales growth"

quietly: tabulate sic2, gen(sicdum)

merge m:1 cty ult_sic year using "TITC_IMPORT_RATIOS.dta"
table year _merge
drop if _merge!=3
drop _merge

****************************************************
*****DO MICRODATA REGRESSIONS
****************************************************


*OLS (col 1)
qui: ivreg2 dlpat5 chinatoctychg cyy*, cluster(max_cult year) partial(cyy*)
estimates store OLS_pat
unique bvd if e(sample)
estadd scalar Nfirms = r(unique)
estadd scalar Nclust = e(N_clust1)

qui: ivreg2 d5sale chinatoctychg cyy*  , cluster(max_cult year) partial(cyy*) 
estimates store OLS_sale
unique bvd if e(sample)
estadd scalar Nfirms = r(unique)
estadd scalar Nclust = e(N_clust1)

*baseline IV (col 2)
qui: ivreg2 dlpat5 (chinatoctychg=chinatouschg) cyy* , cluster(max_cult year) partial(cyy*) first savefirst savefprefix(firstIVbase)
estimates store IVbase_pat
matrix firstmat = e(first)
scalar Ffirst = firstmat[15,1]
estadd scalar Fstat = Ffirst
estadd scalar Nclust = e(N_clust1)
unique bvd if e(sample)
estadd scalar Nfirms = r(unique)

qui: ivreg2 d5sale (chinatoctychg=chinatouschg) cyy* , cluster(max_cult year) partial(cyy*)  first savefirst savefprefix(firstIVbase)
estimates store IVbase_sale
matrix firstmat = e(first)
scalar Ffirst = firstmat[15,1]
estadd scalar Fstat = Ffirst
unique bvd if e(sample)
estadd scalar Nfirms = r(unique)
estadd scalar Nclust = e(N_clust1)

*IV with 2-digit sector dummies (col 3)
qui: ivreg2 dlpat5 (chinatoctychg=chinatouschg) cyy* sicdum* , cluster(max_cult year) partial(cyy* sicdum*) first savefirst savefprefix(first_IVsic_pat)
estimates store IVsic_pat
matrix firstmat = e(first)
scalar Ffirst = firstmat[15,1]
estadd scalar Fstat = Ffirst
unique bvd if e(sample)
estadd scalar Nfirms = r(unique)
estadd scalar Nclust = e(N_clust1)

qui: ivreg2 d5sale (chinatoctychg=chinatouschg) cyy*  sicdum* , cluster(max_cult year) partial(cyy* sicdum*)  first savefirst savefprefix(first_IVsic_sale)
estimates store IVsic_sale
matrix firstmat = e(first)
scalar Ffirst = firstmat[15,1]
estadd scalar Fstat = Ffirst
unique bvd if e(sample)
estadd scalar Nfirms = r(unique)
estadd scalar Nclust = e(N_clust1)

********
******** Output the regressions
********

log using "Table_1.txt", text
estout OLS_pat IVbase_pat IVsic_pat, cells(b se(par) _star) starlevels(* 0.1 ** 0.05  *** 0.01) stats(N Nfirms Nclust Fstat)
estout OLS_sale IVbase_sale IVsic_sale, cells(b se(par) _star) starlevels(* 0.1 ** 0.05  *** 0.01) stats(N Nfirms Nclust Fstat)
log close 

estimates clear

erase "titc.dta"
erase "TITC_IMPORT_RATIOS.dta"
erase "COMTRADE_NBER_INT.dta"
erase "COMTRADE_NBER_INT_baseyear.dta"
erase "TITC_aggs.dta"
erase "TITC_aggs_baseyear.dta"
