#######################################################
# Trapped Factors and China's Impact on Global Growth
# Bloom, Romer, Terry, Van Reenen
# Code to replicate Table 2
# Questions to stephenjamesterry@gmail.com
######################################################

rm(list=ls())
library(pso)
library(matlab)
set.seed(1)

assign('numyrs',10)
assign('growth.target',((1+0.02)^numyrs) - 1)
assign('IoverY.open.target',0.035)
assign('IoverY.fully.mobile.target',0.084)
assign('sigma',1); #flow utility u^(1-sigma) / (1-sigma)
assign('alpha',2/3); #share on human capital
assign('human.cap.ratio',2.96); # from Barro-Lee Data
assign('beta',(1/1.02)^numyrs);  #discount rate
assign('H',.0185);  #northern human capital endowment
assign('Hstar',H*human.cap.ratio);  #southern human capital endowment
assign('rho',0.5); #parameter of innovation production function
assign('gamma',1/rho); #parameter of innovation cost function
assign('phi',.1725);  #proportion of goods off of trade restriction, initialization only
shock = 0.075; # for initialization of phi.prime only 
assign('phi.prime',phi*shock); #post-shock trade parameter, initialization only
assign('eta',1); #eta=1 is fully internal marginal costs, eta=0 is external
assign('n',2); #number of firms (must be even for cross-sectional experiment to work properly)

#some useful constants, same name as in paper
assign('OMEGA',alpha* ( 1 - alpha ) ^ ( ( 2 - alpha ) / alpha ))
assign('PSI',( 1 - alpha ) ^ ( ( alpha - 1 ) / ( 2 - alpha ) ))   

#now, setup the transition path solution
assign('T',7); #of periods assumed for full transition
assign('A1',1); #assumed starting number of varieties
errortol=10^(-15)

calibration = function(IN,eval.dum){
H=IN[1]
phi=IN[2]
shock=IN[3]

assign('phi.prime',phi*shock)
assign('Hstar',human.cap.ratio*H)

##################
##################BGP growth rates
##################

g.open.fun = function(g) {
  

    LHS = g ^ ( gamma - 1) 
    RHS = ( beta ^ ( 1 / alpha ) ) * OMEGA 
    RHS = RHS * ( H + ( PSI ^ ( 1 / alpha ) ) * Hstar * ( ( ( phi * H ) / ( g * Hstar ) ) ^ ( 1 / ( 2 - alpha ) ) ) )
    
    RHS = RHS * ( 1 + g )^(- sigma / alpha )
  
    obj = (LHS - RHS)^2
  
  
}

#use g.open.fun to compute BGP growth rate for phi
g.open.guess=.02
g.open.list = optim(g.open.guess,g.open.fun,gr=NULL,method="Brent",lower=0,upper=1)
g.open = g.open.list$par
assign('g.open',g.open)
q.open = PSI * ( ( phi * H ) / ( g.open * Hstar ) ) ^ ( alpha / (2 - alpha ) ) 
assign('q.open',q.open)
intrate.open = (beta^-1) * (1+g.open)^sigma - 1
assign('intrate.open',intrate.open)
eqbm.check.phi = (q.open <=1)

#use g.open.fun to compute BGP growth rate for phi.prime
phi.orig=phi
assign('phi',phi.prime)
g.shock.mobile.list=optim(g.open,g.open.fun,gr=NULL,method="Brent",lower=0,upper=5)
g.shock.mobile=g.shock.mobile.list$par
assign('g.shock.mobile',g.shock.mobile)
q.shock.mobile = PSI * ( ( phi * H ) / ( g.shock.mobile * Hstar ) ) ^ ( alpha / (2 - alpha ) ) 
intrate.shock.mobile = (beta^-1) * (1+g.shock.mobile)^sigma - 1
assign('intrate.shock.mobile',intrate.shock.mobile)
assign('q.shock.mobile',q.shock.mobile)
assign('phi',phi.orig)
eqbm.check.phi.prime = (q.shock.mobile <=1)

##################
##################calibration moments for imports/gross output
##################

#pre-shock
Iphi = phi; Iphi.min1 = Iphi/(1+g.open); pIphi = q.open * (1+intrate.open); xIphi = (1-alpha)^(1/alpha) * H * (pIphi^(-1/alpha));
Rphi = (1-phi); Rphi.min1 = Rphi/(1+g.open); pRphi = (1+intrate.open); xRphi = (1-alpha)^(1/alpha) * H * (pRphi^(-1/alpha));
Mphi = g.open; Mphi.min1 = Mphi/(1+g.open); pMphi = (1+intrate.open)/(1-alpha); xMphi = (1-alpha)^(1/alpha) * H * (pMphi^(-1/alpha));
Yphi = (H^alpha) * (  Iphi.min1 * (xIphi^(1-alpha)) + Rphi.min1 * (xRphi^(1-alpha)) + Mphi.min1 * (xMphi^(1-alpha))         )
Ygphi = Yphi + Mphi.min1*pMphi*xMphi + Iphi.min1*pIphi*xIphi + Rphi.min1*pRphi*xRphi
IoverY.open = (Iphi.min1*pIphi*xIphi)/(Yphi)

#post-shock
Iphi.prime = phi.prime; Iphi.prime.min1 = Iphi.prime/(1+g.shock.mobile); pIphi.prime = q.shock.mobile * (1+intrate.shock.mobile); xIphi.prime = (1-alpha)^(1/alpha) * H * (pIphi.prime^(-1/alpha));
Rphi.prime = (1-phi.prime); Rphi.prime.min1 = Rphi.prime/(1+g.shock.mobile); pRphi.prime = (1+intrate.shock.mobile); xRphi.prime = (1-alpha)^(1/alpha) * H * (pRphi.prime^(-1/alpha));
Mphi.prime = g.shock.mobile; Mphi.prime.min1 = Mphi.prime/(1+g.shock.mobile); pMphi.prime = (1+intrate.shock.mobile)/(1-alpha); xMphi.prime = (1-alpha)^(1/alpha) * H * (pMphi.prime^(-1/alpha));
Yphi.prime = (H^alpha) * (  Iphi.prime.min1 * (xIphi.prime^(1-alpha)) + Rphi.prime.min1 * (xRphi.prime^(1-alpha)) + Mphi.prime.min1 * (xMphi.prime^(1-alpha))         )
Ygphi.prime = Yphi.prime + Mphi.prime.min1*pMphi.prime*xMphi.prime + Iphi.prime.min1*pIphi.prime*xIphi.prime + Rphi.prime.min1*pRphi.prime*xRphi.prime
IoverY.fully.mobile = (Iphi.prime.min1*pIphi.prime*xIphi.prime)/(Yphi.prime)

error1 = (growth.target-g.open)/growth.target
error2 = (IoverY.open.target-IoverY.open)/IoverY.open.target
error3 = (IoverY.fully.mobile.target-IoverY.fully.mobile)/IoverY.fully.mobile.target

error = error1^2 + error2^2 + error3^2

cat("H = ",H,"phi = ",phi,"phi.prime = ",phi.prime,"error = ",error,"\n")

#eqbm check for phi BGP
eqbm.check.phi = (q.open<=1)

#eqbm check for phi.prime BGP
eqbm.check.phi.prime = (q.shock.mobile<=1)

eqbm.check.BGP = eqbm.check.phi * eqbm.check.phi.prime

if (eval.dum==0){return(error)}

eval.vec = c(g.open,q.open,intrate.open,g.shock.mobile,q.shock.mobile,intrate.shock.mobile,eqbm.check.BGP)

if (eval.dum==1){return(eval.vec)}

}

#perform the calibration exercise, using the BGP
guess=c(.035,.0795,1.28); #H,phi,phi.prime/phi
lb=c(.001,.01,1); ub=c(5,.2,4)
BGPcalibration=psoptim(guess,calibration,lower=lb,upper=ub,eval.dum=0,control=list(abstol=1e-18,maxit=2000))

#extract the calibrated parameters
H = BGPcalibration$par[1]; assign('H',H); Hstar = H * human.cap.ratio; assign('Hstar',Hstar)
phi = BGPcalibration$par[2]; assign('phi',phi)
phi.prime = BGPcalibration$par[2]*BGPcalibration$par[3]; assign('phi.prime',phi.prime)

#extract the growth, exchange, and interest rates
BGPvec = calibration(BGPcalibration$par,eval.dum=1)

assign('g.phi',BGPvec[1])
assign('q.phi',BGPvec[2])
assign('r.phi',BGPvec[3])
assign('g.phi.prime',BGPvec[4])
assign('q.phi.prime',BGPvec[5])
assign('r.phi.prime',BGPvec[6])

sink(file="Table_2.txt")
cat(' ','\n')
cat("########### Table 2 Values",'\n')
cat("Imports to GDP (Pre Shock, Post Shock)",100*IoverY.open.target,100*IoverY.fully.mobile.target,'\n')
cat("Growth Rate (Pre Shock, Post Shock)",100*((1+g.phi)^(1/numyrs)-1),100*((1+g.phi.prime)^(1/numyrs)-1),'\n')
cat("Southern Terms of trade (Pre Shock, Post Shock)",q.phi,q.phi.prime,'\n')
cat(' ','\n')
sink(file=NULL)

